import numpy as np

import matplotlib.pyplot as plt


def draw_brace(ax, xspan, text):
    """Draws an annotated brace on the axes."""
    # https://stackoverflow.com/a/53383764/1889400
    xmin, xmax = xspan
    xspan = xmax - xmin
    ax_xmin, ax_xmax = ax.get_xlim()
    xax_span = ax_xmax - ax_xmin
    ymin, ymax = ax.get_ylim()
    yspan = ymax - ymin
    resolution = int(xspan / xax_span * 100) * 2 + 1  # guaranteed uneven
    beta = 300.0 / xax_span  # the higher this is, the smaller the radius

    x = np.linspace(xmin, xmax, resolution)
    x_half = x[: resolution // 2 + 1]
    y_half_brace = 1 / (1.0 + np.exp(-beta * (x_half - x_half[0]))) + 1 / (
        1.0 + np.exp(-beta * (x_half - x_half[-1]))
    )
    y = np.concatenate((y_half_brace, y_half_brace[-2::-1]))
    y = ymin + (0.05 * y - 0.01) * yspan  # adjust vertical position

    ax.autoscale(False)
    ax.plot(x, y, color="black", lw=3)

    ax.text(
        (xmax + xmin) / 2,
        ymin + 0.07 * yspan,
        text,
        ha="center",
        va="bottom",
        fontsize=20,
    )


if __name__ == "__main__":
    plt.xkcd()
    fig, ax = plt.subplots(figsize=(12, 12))
    plt.rcParams["font.size"] = 30
    ax.spines["top"].set_visible(False)
    ax.spines["right"].set_visible(False)
    xlim = ax.get_xlim()
    ylim = ax.get_ylim()

    ax.plot(
        (xlim[0] + 0.02, xlim[1] - 0.02),
        [0.49, 0.49],
        linestyle="dashed",
        lw=4,
        color="C0",
    )
    ax.plot(
        [0.5, 0.5],
        (ylim[0] + 0.02, ylim[1] - 0.02),
        linestyle="dashed",
        lw=4,
        color="C0",
    )

    ax.text(0.06, 0.8, "COMPUTE BOUND")
    ax.text(
        0.06, 0.65, "- Grid Search\n- Random Forest\n- cross_val_score\n…", fontsize=16
    )

    ax.text(0.6, 0.25, "MEMORY BOUND")

    ax.set(
        ylabel="MODEL SIZE",
        xticks=[],
        yticks=[],
        xlim=xlim,
        ylim=ylim,
        title="DIMENSIONS OF SCALE",
    )
    ax.set_xlabel(xlabel="DATA SIZE", labelpad=10, loc="right", fontsize=20)
    ax.set_ylabel("MODEL SIZE", labelpad=10, fontsize=20)

    draw_brace(ax, (0.01, 0.49), "FITS IN RAM")
    plt.savefig("source/images/dimensions_of_scale.svg")
    plt.savefig("source/images/dimensions_of_scale.png")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# dask-ml documentation build configuration file, created by
# sphinx-quickstart on Fri Jun  9 09:41:40 2017.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import packaging.version

from dask_ml import __version__ as version

# import sys
# sys.path.insert(0, os.path.abspath('.'))


# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.mathjax",
    "sphinx.ext.intersphinx",
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.extlinks",
    "IPython.sphinxext.ipython_console_highlighting",
    "IPython.sphinxext.ipython_directive",
    "numpydoc",
]

intersphinx_mapping = {
    "python": ("https://docs.python.org", None),
    "sklearn": ("https://scikit-learn.org/stable/", None),
    "dask": ("https://docs.dask.org/en/latest/", None),
    "distributed": ("https://distributed.dask.org/en/latest/", None),
    "dask_glm": ("https://dask-glm.readthedocs.io/en/latest/", None),
}

numpydoc_class_members_toctree = False
autodoc_default_flags = ["members", "inherited-members"]
autosummary_generate = True
templates_path = ["templates"]

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = ".rst"

# The master toctree document.
master_doc = "index"

# General information about the project.
project = "dask-ml"
copyright = "2017, Dask developers"
author = "Dask developers"

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
# version = ""

release = packaging.version.parse(version).base_version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ["build", "**.ipynb_checkpoints"]
exclude_trees = ["_build", "includes"]

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "default"

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output ----------------------------------------------


html_theme = "dask_sphinx_theme"
html_theme_options = {"logo_only": True}
# html_theme_path = [dask_sphinx_theme.get_html_theme_path()]

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]


# -- Options for HTMLHelp output ------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = "dask-mldoc"


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',
    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',
    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',
    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, "dask-ml.tex", "dask-ml Documentation", "Dask developers", "manual")
]


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [(master_doc, "dask-ml", "dask-ml Documentation", [author], 1)]


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        master_doc,
        "dask-ml",
        "dask-ml Documentation",
        author,
        "dask-ml",
        "One line description of project.",
        "Miscellaneous",
    )
]


extlinks = {
    "issue": ("https://github.com/dask/dask-ml/issues/%s", "GH#"),
    "pr": ("https://github.com/dask/dask-ml/pull/%s", "GH#"),
}



import os
from codecs import open

from setuptools import find_packages, setup

here = os.path.dirname(__file__)


# Get the long description from the README file
with open(os.path.join(here, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()

install_requires = [
    "dask[array,dataframe]>=2.4.0",
    "distributed>=2.4.0",
    "numba>=0.51.0",
    "numpy>=1.20.0",
    "pandas>=0.24.2",
    "scikit-learn>=1.0.0",
    "scipy",
    "dask-glm>=0.2.0",
    "multipledispatch>=0.4.9",
    "packaging",
]

# Optional Requirements
doc_requires = ["sphinx", "numpydoc", "sphinx-rtd-theme", "nbsphinx", "sphinx-gallery"]
test_requires = [
    "black",
    "coverage",
    "flake8",
    "isort",
    "pytest",
    "pytest-cov",
    "pytest-mock",
]
dev_requires = doc_requires + test_requires
xgboost_requires = ["dask-xgboost", "xgboost"]
complete_requires = xgboost_requires

extras_require = {
    "docs": doc_requires,
    "test": test_requires,
    "dev": dev_requires,
    "xgboost": xgboost_requires,
    "complete": complete_requires,
}

setup(
    name="dask-ml",
    description="A library for distributed and parallel machine learning",
    long_description=long_description,
    url="https://github.com/dask/dask-ml",
    author="Tom Augspurger",
    author_email="taugspurger@anaconda.com",
    license="BSD",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Database",
        "Topic :: Scientific/Engineering",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    packages=find_packages(exclude=["docs", "tests", "tests.*", "docs.*"]),
    use_scm_version=True,
    setup_requires=["setuptools_scm"],
    install_requires=install_requires,
    extras_require=extras_require,
    python_requires=">=3.6",
)
